#!/bin/bash

. /etc/socket-relay/path.conf
. /etc/socket-relay/incoming.conf

. $INCOMING_PATH/common_functions

TAG=incoming

DEBUG=""
ADDR_TYPE=exec
#ADDR_TYPE=system

if [ "$SOCAT_DEBUG" != "" ]; then
    SOCAT_OPTIONS_CTRL=$SOCAT_DEBUG
fi

if [ $SOCAT_CTRL_POLL_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_CTRL="$SOCAT_OPTIONS_CTRL -t $SOCAT_CTRL_POLL_TIMEOUT"
fi

if [ $SOCAT_CTRL_INACTIVITY_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_CTRL="$SOCAT_OPTIONS_CTRL -T $SOCAT_CTRL_INACTIVITY_TIMEOUT"
fi

usage()
{
    echo "Usage: $0 [start {debug} | stop {force} ]" >&2
    exit 1
}

signal_hup()
{
    logger -i -t $TAG "SIGNAL HUP"
    do_stop
    exit 1
}

signal_int()
{
    logger -i -t $TAG "SIGNAL INT"
    do_stop
    exit 1
}

signal_term()
{
    logger -i -t $TAG "SIGNAL TERM"
    do_stop
    exit 1
}

config_chat()
{
    echo "ABORT BUSY" > $INCOMING_CHAT_CONFIG_FILE
    echo "ABORT 'NO CARRIER'" >> $INCOMING_CHAT_CONFIG_FILE
    echo "ABORT VOICE" >> $INCOMING_CHAT_CONFIG_FILE
    echo "ABORT 'NO DIALTONE'" >> $INCOMING_CHAT_CONFIG_FILE
    echo "ABORT 'NO DIAL TONE'" >> $INCOMING_CHAT_CONFIG_FILE
    echo "ABORT 'NO ANSWER'" >> $INCOMING_CHAT_CONFIG_FILE
    echo "ABORT DELAYED" >> $INCOMING_CHAT_CONFIG_FILE
    echo "TIMEOUT $MODEM_CHAT_TIMEOUT" >> $INCOMING_CHAT_CONFIG_FILE
    echo "'' ATZ" >> $INCOMING_CHAT_CONFIG_FILE
    echo "OK '$MODEM_AT_INIT'" >> $INCOMING_CHAT_CONFIG_FILE
    echo "OK ATS7=$MODEM_ATS7" >> $INCOMING_CHAT_CONFIG_FILE
    echo "OK ATS30=$MODEM_ATS30" >> $INCOMING_CHAT_CONFIG_FILE
    echo "OK-AT-OK ATDT$TELEPHONE" >> $INCOMING_CHAT_CONFIG_FILE
    echo "CONNECT '\c'" >> $INCOMING_CHAT_CONFIG_FILE
}

config_stty()
{
    BAUDRATE=$(echo $TERMIOS_BAUDRATE | sed -e 's/b//')

    echo "$BAUDRATE" > $INCOMING_STTY_CONFIG_FILE
}

do_start()
{
    local pid

    logger -i -t $TAG "starting"

    if [ -f $PID_INCOMING_CTRL ]; then
        pid=$(cat $PID_INCOMING_CTRL)
        logger -i -t $TAG "socat is already running as $pid"
        exit 1
    fi

    init_modem
    config_chat
    config_stty

    if [ "$ON_DEMAND" = "yes" ]; then
        mkdir $ON_DEMAND_STAT 2> /dev/null
    else
        rmdir $ON_DEMAND_STAT 2> /dev/null
    fi

    case $DEBUG in
        normal)
            socat $SOCAT_OPTIONS_CTRL tcp-l:${CTRL_PORT},reuseaddr,fork ${ADDR_TYPE}:"bash ${INCOMING_PATH}/incoming_modem.sh" &
            if [ $? -ne 0 ]; then
                logger -i -t $TAG "socat failed with $ret"
                exit 1
            fi
            echo -n $! > $PID_INCOMING_CTRL
            ;;
        debug)
            socat $SOCAT_OPTIONS_CTRL tcp-l:${CTRL_PORT},reuseaddr,fork ${ADDR_TYPE}:"bash -x ${INCOMING_PATH}/incoming_modem.sh"
            if [ $? -ne 0 ]; then
                logger -i -t $TAG "socat failed with $ret"
                exit 1
            fi
            echo -n $! > $PID_INCOMING_CTRL
            ;;
        stdio)
            socat $SOCAT_OPTIONS_CTRL tcp-l:${CTRL_PORT},reuseaddr,fork stdout
            ;;
        *)
            usage
            ;;
    esac
}

do_stop()
{
    local pid

    logger -i -t $TAG "stopping"

    release_disconnect_modem

    if [ -f $PID_INCOMING_CTRL ]; then
        pid=$(cat $PID_INCOMING_CTRL)
        logger -i -t $TAG "killing incoming[${pid}]"
        kill $pid 2> /dev/null
        rm -f $PID_INCOMING_CTRL 2> /dev/null
    else
        logger -i -t $TAG "socat is not running"
    fi
}

case $1 in
    start)
        if [ $# -eq 2 ]; then
            DEBUG=$2
        else
            DEBUG=normal
        fi

        do_start
        ;;
    stop)
        do_stop
        if [ "$2" = "force" ]; then
            killall -9 socat &> /dev/null
            rm -f /var/run/${TTYDEV}.lock 2> /dev/null
            rm -f $PID_INCOMING_DATA 2> /dev/null
        fi
        ;;
    *)
        usage
        ;;
esac

exit 0
