#!/bin/bash

. /etc/socket-relay/path.conf
. /etc/socket-relay/outcalling.conf

TAG=agi_call
CONNECTIONS=/etc/socket-relay/connections.lst
OUTPUT=/tmp/agi_call.out
TIMEOUT=$CALL_TIMEOUT

cp /dev/null $OUTPUT

while read line; do
    if [ -n $line ]; then
        break;
    fi

    echo $line >> $OUTPUT
    
    AGI=$(echo -n $line | awk -F":" '{print $1}')
    VAL=$(echo -n $line | awk -F":" '{print $2}' | sed -e 's/^ *//')

    if [ "$AGI" = "agi_dnid" ]; then
        DNID=$VAL
    fi

    if [ "$AGI" = "agi_extension" ]; then
        if [ "$DNID" = "unknown" ]; then
            DNID=$VAL
        fi
    fi
done

logger -t $TAG "dnid[${DNID}]"

if [ "$DNID" = "h" ]; then
    $OUTCALLING_PATH/outcalling.sh stop

    echo "SUCCESS"
    logger -t $TAG "SUCCESS"
    exit 0
else
    while read list; do
        _DNID=$(echo -n $list | awk -F":" '{print $1}')
        IPADDR=$(echo -n $list | awk -F":" '{print $2}')

        if [ "$DNID" = "$_DNID" ]; then
            logger -t $TAG "try connection to $IPADDR"
            if [ "$SYNC_CONNECT" = "yes" ]; then
                logger -t $TAG "sync connection"
                $OUTCALLING_PATH/outcalling.sh start $IPADDR > /dev/null 2>&1
                if [ $? -eq 0 ]; then
                    while [ ! -d $OUTCALLING_RECV_ACK -a $TIMEOUT -ne 0 ]; do
                        TIMEOUT=$(($TIMEOUT-1))
                        sleep 1
                    done
                    if [ -f $PID_OUTCALLING_DATA ]; then
                        echo "SUCCESS"
                        logger -t $TAG "SUCCESS"
                        exit 0
                    fi
                fi
                break;
            else
                logger -t $TAG "async connection"
                $OUTCALLING_PATH/outcalling.sh start $IPADDR > /dev/null 2>&1 &
                echo "SUCCESS"
                logger -t $TAG "SUCCESS"
                exit 0
            fi
        fi
    done < $CONNECTIONS

    echo "HANGUP"
    logger -t $TAG "HANGUP"
    exit 1
fi
