#!/bin/bash

. /etc/socket-relay/path.conf
. /etc/socket-relay/outcalling.conf

. $OUTCALLING_PATH/common_functions

TAG=outcalling

DEBUG=""
CTRL_IP=""
DATA_IP=""
ADDR_TYPE=exec
#ADDR_TYPE=system

if [ "$SOCAT_DEBUG" != "" ]; then
    SOCAT_OPTIONS_CTRL=$SOCAT_DEBUG
fi

if [ $SOCAT_CTRL_POLL_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_CTRL="$SOCAT_OPTIONS_CTRL -t $SOCAT_CTRL_POLL_TIMEOUT"
fi

if [ $SOCAT_CTRL_INACTIVITY_TIMEOUT -ne 0 ]; then
    SOCAT_OPTIONS_CTRL="$SOCAT_OPTIONS_CTRL -T $SOCAT_CTRL_INACTIVITY_TIMEOUT"
fi

usage()
{
    echo "Usage: $0 [start <ip address> {debug} | stop {force} ]" >&2
    exit 1
}

config_chat()
{
    echo "ABORT BUSY" > $OUTCALLING_CHAT_CONFIG_FILE
    echo "ABORT 'NO CARRIER'" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "ABORT VOICE" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "ABORT 'NO DIALTONE'" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "ABORT 'NO DIAL TONE'" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "ABORT 'NO ANSWER'" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "ABORT DELAYED" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "TIMEOUT $MODEM_CHAT_TIMEOUT" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "'' ATZ" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "OK '$MODEM_AT_INIT'" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "OK ATS0=$MODEM_ATS0" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "OK ATS30=$MODEM_ATS30" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "RING ATA" >> $OUTCALLING_CHAT_CONFIG_FILE
    echo "CONNECT '\c'" >> $OUTCALLING_CHAT_CONFIG_FILE
}

config_stty()
{
    BAUDRATE=$(echo $TERMIOS_BAUDRATE | sed -e 's/b//')

    echo "$BAUDRATE" > $OUTCALLING_STTY_CONFIG_FILE
}

do_start()
{
    local pid
    local ret

    logger -i -t $TAG "starting"

    rmdir $OUTCALLING_RECV_ACK 2> /dev/null

    if [ $NOT_USE_MGETTY = "no" ]; then
        stop $TTYDEV
    fi

    if [ -f $PID_OUTCALLING_DATA ]; then
        pid=$(cat $PID_OUTCALLING_DATA)
        logger -i -t $TAG "socat is already running as $pid"
        if [ "$STOP_ALREADY_RUNNING" = "yes" ]; then
            exit 1
        else
            do_stop
        fi
    fi

    init_modem
    config_chat
    config_stty

    case $DEBUG in
        normal)
            socat $SOCAT_OPTIONS_CTRL tcp:${CTRL_IP}:${CTRL_PORT} ${ADDR_TYPE}:"bash ${OUTCALLING_PATH}/outcalling_modem.sh $DATA_IP" &
            ret=$?
            if [ $ret -ne 0 ]; then
                logger -i -t $TAG "socat failed with $ret"
                exit 1
            fi
            ;;
        debug)
            socat $SOCAT_OPTIONS_CTRL tcp:${CTRL_IP}:${CTRL_PORT} ${ADDR_TYPE}:"bash -x ${OUTCALLING_PATH}/outcalling_modem.sh $DATA_IP"
            ret=$?
            if [ $ret -ne 0 ]; then
                logger -i -t $TAG "socat failed with $ret"
                exit 1
            fi
            ;;
        stdio)
            socat $SOCAT_OPTIONS_CTRL tcp:${CTRL_IP}:${CTRL_PORT} stdin
            ;;
        *)
            usage
            ;;
    esac
}

do_stop()
{
    local pid

    logger -i -t $TAG "stopping"

    if [ -f $PID_OUTCALLING_DATA ]; then
        pid=$(cat $PID_OUTCALLING_DATA)
        echo -n disconnect > $PID_OUTCALLING_DATA
        sleep $DISCONNECT_WAIT
        logger -i -t $TAG "killing outcalling_modem[${pid}]"
        kill $pid 2> /dev/null
        rm -f $PID_OUTCALLING_DATA 2> /dev/null
    else
        logger -i -t $TAG "socat is not running"
    fi
}

check_ip()
{
    local ip=$1
    ip_check=$(echo ${ip} | egrep "^(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$")

    if [ "$ip_check" = "" ] ; then
	logger -i -t $TAG  $ip is not IP Address.
	return 1
    fi

    return 0
}

case $1 in
    start)
        if [ $# -lt 2 ]; then
            usage
        fi

        check_ip $2
        if [ $? -ne 0 ]; then
            usage
        fi

        CTRL_IP=$2
        DATA_IP=$CTRL_IP

        if [ $# -eq 3 ]; then
            DEBUG=$3
        else
            DEBUG=normal
        fi

        do_start
        ;;
    stop)
        do_stop

        if [ "$2" = "force" ]; then
            killall -9 socat &> /dev/null
        fi
        ;;
    *)
        usage
        ;;
esac

exit 0
